@@ -9,12 +9,14 @@ class BandgapAnalyzer():
         self.xes_calc_spectra = {}
         self.xas_exp_spectra = {}
         self.xas_calc_spectra = {}
-        self.figsize = (14, 8)
+        self.figsize = (12, 6)
         self.xes_xlims = None
         self.xas_xlims = None
         self.subplot_labels = [['XES' , 'XAS'], ['2nd der.', '2nd der.']]
         self.xes_arrow = None
         self.xas_arrow = None
+        self.xes_line = None
+        self.xas_line = None
         self.title = None
 
         self.gs_onsets = []
@@ -28,6 +30,12 @@ class BandgapAnalyzer():
     def __normalize(self, list):
         abs_max = max(max(list), -min(list))
         return [x/abs_max for x in list]
+    
+    
+    def __find_onset(self, path, binding, fermi):
+        df = pd.read_csv(path, sep='\\s+', header=None, names=['energy', 'intensity1', 'intensity2', 'intensity3'])
+        onset = min(df.loc[df['intensity1']>0]['energy']) + (binding + fermi) * ryd_to_ev
+        return onset
 
     """
     Load experimental spectra
@@ -40,9 +48,9 @@ class BandgapAnalyzer():
         Type of the spectra. Must be either 'xes' or 'xas'
     names : list of str
         Names of the spectra
-    skiprows : int
+    skiprows : int, optional
         Number of rows to skip in the beginning of the file
-    sep : str
+    sep : str, optional
         Separator used in the file
     """
     def load_exp_spectra(self, path, type, names, skiprows=2, sep=','):        
@@ -120,6 +128,14 @@ class BandgapAnalyzer():
         # take second derivative
         exp_spectra[name][f'{name}_smoothed_2nd'] = np.gradient(np.gradient(y_smoothed, x), x)
 
+
+        # make sure onset region includes at least one data point
+        if onset_region:
+            if onset_region[0] > onset_region[1]:
+                raise ValueError('The start of the onset region must be smaller than the end.')
+            if onset_region[1] < min(x) or onset_region[0] > max(x):
+                raise ValueError('The onset region is outside of the energy range.')
+
         if show:
             # plot the smoothed data
             if onset_region:
@@ -200,6 +216,36 @@ class BandgapAnalyzer():
             raise ValueError('type must be either "xes" or "xas"')
 
 
+    """
+    Add a vertical line to the plot with energy label
+
+    Parameters
+    ----------
+    type : str
+        Type of the spectra. Must be either 'xes' or 'xas'
+    energy : float
+        Energy of the line
+    linestyle : str, optional
+        Linestyle of the line
+    color : str, optional
+        Color of the line
+    linewidth : float, optional
+        Width of the line
+    label : bool, optional
+        If True, the energy will be shown as a label
+    xytext : tuple of float, optional
+        Tuple containing the x and y coordinates of the label
+    """
+
+    def add_line(self, type, energy, linestyle='--', color='black', linewidth=1, label=True, xytext=None):
+        if type == 'xes':
+            self.xes_line = (energy, linestyle, color, linewidth, label, xytext)
+        elif type == 'xas':
+            self.xas_line = (energy, linestyle, color, linewidth, label, xytext)
+        else:
+            raise ValueError('type must be either "xes" or "xas"')
+
+
     """
     Plot the spectra in four plots with the XES and XAS in the top row and their 2nd derivatives in the bottom row.
 
@@ -257,7 +303,7 @@ class BandgapAnalyzer():
                 
 
                 # show text in corner
-                ax.text(0.11/self.figsize[0], 1 - (1+i)*0.08/self.figsize[1], self.subplot_labels[i][j], size=12, ha='left', va='top', color='white', bbox=dict(facecolor='black', edgecolor='none', pad=3), transform=ax.transAxes)
+                ax.text(0.11/self.figsize[0], 1 - (1+i)*0.08/self.figsize[1], self.subplot_labels[i][j], size=11, ha='left', va='top', color='white', bbox=dict(facecolor='black', edgecolor='none', pad=3), transform=ax.transAxes)
                 
                 if i == 0: # spectra subplots
                     ax.legend(loc='upper right')                    
@@ -271,17 +317,59 @@ class BandgapAnalyzer():
                 xy, xytext, text, text_rot = self.xes_arrow
                 axes[1,0].annotate(text, xy=xy, xytext=xytext, arrowprops=dict(facecolor='black', shrink=0.05), ha='center', va='center', rotation=text_rot)
 
+            if self.xes_line:
+                energy, linestyle, color, linewidth, label, xytext = self.xes_line
+                axes[0,0].axvline(x=energy, color=color, linestyle=linestyle, linewidth=linewidth)
+                axes[1,0].axvline(x=energy, color=color, linestyle=linestyle, linewidth=linewidth)
+                # add energy label
+                if label:
+                    xytext = xytext if xytext else (energy + 0.2, -0.7)
+                    xy = (energy, xytext[1]) if xytext else (energy, -0.7)
+                    axes[1,0].annotate(f'{energy} eV', xy=xy, xytext=xytext, ha='left')
+
             # annotate first XAS peak
             if self.xas_arrow:
                 xy, xytext, text, text_rot = self.xas_arrow
                 axes[1,1].annotate(text, xy=xy, xytext=xytext, arrowprops=dict(facecolor='black', shrink=0.05), ha='center', va='center', rotation=text_rot)
 
-    
-    def __find_onset(self, path, binding, fermi):
-        df = pd.read_csv(path, sep='\\s+', header=None, names=['energy', 'intensity1', 'intensity2', 'intensity3'])
-        onset = min(df.loc[df['intensity1']>0]['energy']) + (binding + fermi) * ryd_to_ev
-        return onset
-    
+            if self.xas_line:
+                energy, linestyle, color, linewidth, label, xytext = self.xas_line
+                axes[0,1].axvline(x=energy, color=color, linestyle=linestyle, linewidth=linewidth)
+                axes[1,1].axvline(x=energy, color=color, linestyle=linestyle, linewidth=linewidth)
+                # add energy label
+                if label:
+                    xytext = xytext if xytext else (energy - 0.2, -0.7)
+                    xy = (energy, xytext[1]) if xytext else (energy, -0.7)
+                    axes[1,1].annotate(f'{energy} eV', xy=xy, xytext=xytext, ha='right')
+
+    """
+    Export 2nd derivative of XES and XAS spectra to csv files
+
+    Parameters
+    ----------
+    path : str
+        Path to the directory where the files will be saved
+    name : str
+        Name of the files (without extension)
+    """
+    def export_2nd_derivative(self, path, name):
+        xes_2nd = pd.DataFrame()
+        for xes_exp_name in self.xes_exp_spectra.keys():
+            if f'{xes_exp_name}_smoothed_2nd' not in self.xes_exp_spectra[xes_exp_name].columns:
+                print(f'No 2nd derivative found for {xes_exp_name}. Run smoothen method first, if you want to export the 2nd derivative.')
+                continue
+            xes_2nd[f'{xes_exp_name}_energy'] = self.xes_exp_spectra[xes_exp_name][f'{xes_exp_name}_energy']
+            xes_2nd[f'{xes_exp_name}_2nd'] = self.xes_exp_spectra[xes_exp_name][f'{xes_exp_name}_smoothed_2nd']
+        xes_2nd.to_csv(f'{path}/{name}_XES_2nd.csv', index=False)
+
+        xas_2nd = pd.DataFrame()
+        for xas_exp_name in self.xas_exp_spectra.keys():
+            if f'{xas_exp_name}_smoothed_2nd' not in self.xas_exp_spectra[xas_exp_name].columns:
+                print(f'No 2nd derivative found for {xas_exp_name}. Run smoothen method first, if you want to export the 2nd derivative.')
+                continue
+            xas_2nd[f'{xas_exp_name}_energy'] = self.xas_exp_spectra[xas_exp_name][f'{xas_exp_name}_energy']
+            xas_2nd[f'{xas_exp_name}_2nd'] = self.xas_exp_spectra[xas_exp_name][f'{xas_exp_name}_smoothed_2nd']
+        xas_2nd.to_csv(f'{path}/{name}_XAS_2nd.csv', index=False)
 
     """
     Load unbrodened spectra to determine core hole shift
@@ -306,7 +394,8 @@ class BandgapAnalyzer():
         self.es_onsets.append(self.__find_onset(f'{dir}/{ES_file}', GS_binding, ES_fermi))
 
     """
-    Calculate the core hole shift using all previously loaded unbrodened spectra
+    Calculate the core hole shift using all previously loaded unbrodened spectra.
+    !Warning this might not be accurate!
 
     Returns
     -------
