@@ -27,9 +27,9 @@ class DOSAnalyzer:
         self.xas_x_lims = None
         self.staggered_spacing = 0.2
         self.title = 'DOS Analyzer'
+        self.figsize = (12, 4)
 
     def __init_plt__(self, show_spectra):
-        self.figsize = (15, 6)
 
         # create figure and axis depending on number of spectra to show
         self.fig, axis = plt.subplots(1, len(show_spectra), figsize=self.figsize, sharey=True)
@@ -116,7 +116,7 @@ class DOSAnalyzer:
         return selected_dos_dfs
     
     def __configure_plot__(self, ax, spectra, x_min, label, fig_count, x_lims=None):
-            ax.text(0.075*fig_count/self.figsize[0], 0.2/self.figsize[1], label, ha='left', va='top', color='white', size=12, bbox=dict(facecolor='black', edgecolor='none', pad=3.0), transform=ax.transAxes)   
+            ax.text(0.055*fig_count/self.figsize[0], 0.0155 + 0.1/self.figsize[1], label, ha='left', va='top', color='white', size=12, bbox=dict(facecolor='black', edgecolor='none', pad=3.0), transform=ax.transAxes)   
             if x_lims is None:
                 x_lims = self.__get_x_lims__(spectra.values())
             ax.set_xlim(x_lims)
@@ -244,11 +244,11 @@ class DOSAnalyzer:
         Name of the spectrum
     spec_type : str
         Type of the spectrum ("XES" or "XAS")
-    skiprows : int
+    skiprows : int, optional
         Number of rows to skip in the file
-    delim : str
+    delim : str, optional
         Delimiter of the file
-    shift : float
+    shift : float, optional
         Energy shift of the spectrum
     """
     def load_spectrum(self, path, name, spec_type, skiprows=1, delim=',', shift=0):
@@ -331,9 +331,9 @@ class DOSAnalyzer:
 
     Parameters
     ----------
-    xes_names : list of str
+    xes_names : list of str, optional
         List of XES DOS names to plot
-    xas_names : list of str
+    xas_names : list of str, optional
         List of XAS DOS names to plot
     """
     def set_active_dos(self, xes_names=[], xas_names=[]):
@@ -372,9 +372,9 @@ class DOSAnalyzer:
 
     Parameters
     ----------
-    xes_x_limits : tuple of float
+    xes_x_limits : tuple of float, optional
         X limits for XES DOS
-    xas_x_limits : tuple of float
+    xas_x_limits : tuple of float, optional
         X limits for XAS DOS
     """
     def set_x_limits(self, xes_x_limits=None, xas_x_limits=None):
@@ -389,8 +389,8 @@ class DOSAnalyzer:
     ----------
     staggered : bool
         Staggered DOS
-    show_spectra : list of str
-        List of spectra to show ("XES" or "XAS")
+    show_spectra : list of str, optional
+        List of spectra to show ("XES" or "XAS"), default shows both
     """
     def plot_dos(self, staggered=False, show_spectra=['XES', 'XAS']):
         if len(show_spectra) == 0:
@@ -463,8 +463,8 @@ class DOSAnalyzer:
         Path to the directory where to save the DOS
     name : str
         Name of the DOS file
-    export_spectra : list of str
-        List of spectra to export ("XES" or "XAS")
+    export_spectra : list of str, optional
+        List of spectra to export ("XES" or "XAS"), default exports both
     """
     def export_dos(self, path, name, export_spectra=['XES', 'XAS']):
         if len(export_spectra) == 0:
